/*
 * Hexagonal Grid
 * Author: niasw
 */

#ifndef _HEXGRID_
#define _HEXGRID_

#ifndef _CMATH_
#define _CMATH_
#include <cmath>
#endif

#ifndef _IOSTREAM_
#define _IOSTREAM_
#include <iostream>
#include <cstdlib>
#endif

namespace hexagon
{

    /***integer type***/

    struct veci
    {
        int* x; //coordinates
        static veci* adj; //adjacents (6 veci)
        static veci adjstart; //start point of traversal

        veci(int x1=0,int x2=0,int x3=0); //construction
        veci(const veci& vy); //copy construction
        ~veci(); //destruction
        inline int& operator[] (int n) const {return x[n];}
        inline veci operator+ (const veci& vo) const
            {return veci(x[0]+vo[0],x[1]+vo[1],x[2]+vo[2]);}
        inline veci operator- (const veci& vo) const
            {return veci(x[0]-vo[0],x[1]-vo[1],x[2]-vo[2]);}
        inline veci operator* (const int k) const
            {return veci(x[0]*k,x[1]*k,x[2]*k);}
        veci& operator= (const veci& vy);
        veci& operator+= (const veci& vy);
        veci& operator-= (const veci& vy);
        friend double d2 (const veci& v);
        friend double d (const veci& v);
        inline double d2 () const {return (x[0]*x[0]+x[1]*x[1]+x[2]*x[2])/2.;}
        double d () const;
        inline bool check() const {return (x[0]+x[1]+x[2]==0);}
        friend veci operator* (const int k, const veci& v);
        friend std::ostream& operator<< (std::ostream& os, const veci& vo);

    private:
        static bool initOK; //whether initialized
    };

    /***double type***/

    struct vecd
    {
        double* x; //coordinates

        vecd(double x1=0,double x2=0,double x3=0); //construction
        vecd(double x=0,double y=0); //convert from Descartes Coordinates
        vecd(const vecd& vy); //copy construction
        vecd(const veci& vy); //copy construction
        ~vecd(); //destruction
        inline double& operator[] (int n) const {return x[n];}
        inline vecd operator+ (const vecd& vo) const
            {return vecd(x[0]+vo[0],x[1]+vo[1],x[2]+vo[2]);}
        inline vecd operator- (const vecd& vo) const
            {return vecd(x[0]-vo[0],x[1]-vo[1],x[2]-vo[2]);}
        inline vecd operator* (const double k) const
            {return vecd(x[0]*k,x[1]*k,x[2]*k);}
        vecd& operator= (const vecd& vy);
        vecd& operator+= (const vecd& vy);
        vecd& operator-= (const vecd& vy);
        friend double d2 (const vecd& v);
        friend double d (const vecd& v);
        inline double d2 () const {return (x[0]*x[0]+x[1]*x[1]+x[2]*x[2])/2.;}
        double d () const;
        inline bool check () const {return (x[0]+x[1]+x[2]==0);}
        friend vecd operator* (const double k, const vecd& v);
        friend std::ostream& operator<< (std::ostream& os, const vecd& vo);
        friend void hexagon2square(double* xy, const vecd& v); //(x1,x2,x3)->(x,y)
    };

    /***Useful Functions***/

    double d2 (const veci& v);
    double d (const veci& v);
    double d2 (const vecd& v);
    double d (const vecd& v);
    std::ostream& operator<< (std::ostream& os, const veci& vo);
    std::ostream& operator<< (std::ostream& os, const vecd& vo);
    veci operator* (const int k, const veci& v);
    vecd operator* (const double k, const vecd& v);
    void hexagon2square(double* xy, const vecd& v);

    /***container***/

    class hexagondev
    {
        veci* nodes;
        int sidelenth;
        int nodes_num; //number of nodes in a triangle partion

    public:
        hexagondev(int sidelength); //construction
        ~hexagondev(); //destruction

        inline veci& operator[] (int order) const {return nodes[order];} //order->vector
        int operator[] (const veci& vector) const; //vector->order
            /* return -1 for 'NOT FOUND IN THIS DEVICE' */

        inline int GetSideLength () {return sidelenth;}
        inline int GetNodeNumber () {return 6*nodes_num+1;}
    };
}

#endif

