/*
 * Main Program for Hexgrid Simulation
 * Author: niasw
 */

#define _IOSTREAM_
#include <iostream>
#include <cstdlib>
#include <fstream>
using namespace std;

#include "hexgridplus.hpp"
using namespace hexagonplus;
#include "updatefunc.hpp"

/*
 * Functions
 */
void Prologue();
void MainBody();
void Epilogue();
void InitAsgn(); //assign initial values (in updatefunc.cpp)

void WriteHead(); //write my rec filehead
void WriteState(); //write current state of device

/*
 * Global Project Variables
 */

extern int devSideLen; //side length of the device
hexagondev dev(devSideLen); //device contains the cells' world
double* var; //variables of the cell
extern double time_step; //time step used in calculation
extern int save_freq; //number of frames per save
extern double STAtime; //start time
extern double ENDtime; //end time
extern int varnum; //number of variables

double nowtime=0; //time in calculation
double nowsave=0; //saving frame counting
ofstream ofs; //file output stream
const char* filename; //output as my record of movie

/*
 * Temporary Variables
 */
 //temporary Descartes Coordinates
double ty1[2];
double ty2[2];
 //iterators
int it0,it1,it2,it3;
 //temporary Value
double* var_; //var next time
double* incvar; //var increase
double* incvar_; //var_ increase
 //Control Variables
char cont='y';
int filelength;

union buffer
{
    char c[8];
    int i[2];
    double d[1];
};

buffer buf;

//--------------Main Procedure------------
int main ( int argc, char *argv[] )
{
    if (devSideLen<0 || varnum<0) {cerr<<"Invalid Input >>> see updatefunc.cpp"<<endl;return -1;}
    if (argc=1) filename=argv[1];
    else filename="SimuData.rec";
    if (argc>1) {cout<<"Usage: DynamicSimu [outputfile.rec]"<<endl;return 0;}
    Prologue();
    InitAsgn();
    MainBody();
    Epilogue();
    return 0;
}

void Prologue()
{
    var=new double[varnum*dev.GetNodeNumber()];
    var_=new double[varnum*dev.GetNodeNumber()];
    incvar=new double[varnum*dev.GetNodeNumber()];
    incvar_=new double[varnum*dev.GetNodeNumber()];
}

void Epilogue()
{
    delete[] var;
    delete[] var_;
    delete[] incvar;
    delete[] incvar_;
    ofs.close();
}

void MainBody()
{
    int it=0;
    ofs.open(filename, ios::trunc | ios::binary);
    ofs.close();
    filelength=0;
    while (cont!='n')
    {
        ofs.open(filename, ios::in | ios::out | ios::binary); //ios::in for no truncating
        WriteHead();
        while (nowtime<ENDtime)
        {
            if (it==save_freq)
            {
                cout<<"Output: time="<<nowtime<<endl;
                WriteState();
                ++nowsave;
                it=0;
            }
            ++it;
            Evolve(var, nowtime, UpdateFunc);
            nowtime += time_step;
        }
        filelength += ofs.tellp();
        ofs.close();
        cout<<"continue?(y/n)"<<endl;
        cin>>cont;
        ENDtime+=100*save_freq*time_step;
    }
}

void WriteHead()
{
    ofs.seekp(0, ios::beg);
    ofs.write("HEX\0",4); //0x00-0x02: HEX 0x03:00
    buf.i[0]=0x40;
    ofs.write(buf.c,4); //0x04-0x07: start site (0x40)
    buf.i[0]=devSideLen;
    ofs.write(buf.c,4); //0x08-0x0b: hexagon side length
    buf.i[0]=varnum;
    ofs.write(buf.c,4); //0x0c-0x0f: number of variables
    buf.d[0]=STAtime;
    ofs.write(buf.c,8); //0x10-0x17: start time
    buf.d[0]=ENDtime;
    ofs.write(buf.c,8); //0x18-0x1f: end time
    buf.d[0]=time_step*save_freq;
    ofs.write(buf.c,8); //0x20-0x27: time per viewing frame
    buf.i[0]=save_freq;
    ofs.write(buf.c,4); //0x28-0x2b: calculating frames per viewing frame
    ofs.write("DynamicSimu by niasw",0x14); //0x2c-0x3f: comment area
    ofs.seekp(0x40 + nowsave * dev.GetNodeNumber() * varnum * 8, ios::beg); //start site
}

void WriteState()
{
    ofs.write((char*)var, dev.GetNodeNumber() * varnum * 8); //write data
}
