/*
 * Update Function and Parameters
 */
#define _IOSTREAM_
#include <iostream>
#include <cstdlib>

#include "hexgridplus.hpp"
using namespace hexagonplus;
#include "updatefunc.hpp"

/*
 * Parameters
 *
 * Time unit = second
 * Length unit = micrometer
 */

double mm=5; // 1 micrometer = 50 units

int devSideLen=60; //side length of the device
int save_freq=100; //number of frames per save
double time_step=0.01; //time step used in calculation
double STAtime=0; //start time
double ENDtime=50; //end time
int varnum=5; //number of variables

/*
 * Automatic Variables
 */

extern hexagondev dev; //device
extern double* var; //variables
extern double nowtime; //current time
static int it0,it1,it2; //iterator
int tor; //order
veci tv; //vector

/***parameters***/
const double m_x=3;
const double d_x=0.005;
const double g_x=0.01;
const double m_l=4;
const double d_l=0.001;
const double g_l=0.03;
const double k_a=1;
const double k_r=0.5;
const double k7=0.5;
const double K7i=1;
const double D_a=1;
const double g_a=0.2;
const double e_l=0.5;
const double k8=1;
const double K0=1;
const double E0=1;
const double Vt=500;
const double g_v=0.3;
const double distdecay=3;
const int distinteg=8;

static double InterFactor(const veci& vec1, const veci& vec2, double value1, double value2)
{
    if (d2(vec1-vec2)==0) return e_l*value2*10/distdecay;
    else return e_l*value2*exp(-d(vec1-vec2)/distdecay)/d2(vec1-vec2);
}

void UpdateFunc(double* return_increase, double t, double* input_var)
{
    double* X = input_var; //LuxI
    double* L = input_var + dev.GetNodeNumber(); //luciferase
    double* A = input_var + 2*dev.GetNodeNumber(); //AHL
    double* V = input_var + 3*dev.GetNodeNumber(); //Luminesensor
    double* E = input_var + 4*dev.GetNodeNumber(); //Illuminance
    double* re_X = return_increase;
    double* re_L = return_increase + dev.GetNodeNumber();
    double* re_A = return_increase + 2*dev.GetNodeNumber();
    double* re_V = return_increase + 3*dev.GetNodeNumber();
    double* re_E = return_increase + 4*dev.GetNodeNumber();

    for (it1=0;it1<dev.GetNodeNumber();++it1)
    {
        re_X[it1]=0;re_L[it1]=0;re_A[it1]=0;re_V[it1]=0;re_E[it1]=0;
        tv=dev[it1];
        for (it2=0;it2<6;++it2)
        {
            tor=dev[tv+veci::adj[it2]];
            if (tor!=-1)
            {
                re_A[it1]+=A[tor]-A[it1]; //sum around - 6 center
            }
        }
        re_A[it1]*=2./3*D_a/mm/mm;
        re_A[it1]+=k7*X[it1]/(K7i+X[it1])-g_a*A[it1];
        re_X[it1]=m_x*(d_x+A[it1]*A[it1])/(k_a+A[it1]*A[it1])/(1+k_r*V[it1])-g_x*X[it1];
        re_L[it1]=m_l*(d_l+A[it1]*A[it1])/(k_a+A[it1]*A[it1])/(1+k_r*V[it1])-g_l*L[it1];
        E[it1]=IntegrateAround(dev,L,distinteg,it1,InterFactor);
        re_V[it1]=k8/(1+(E0+E[it1])/K0/E[it1])/(1+(E0+E[it1])/K0/E[it1])*(Vt-2*V[it1])-g_v*V[it1];

        //std::cout<<X[0]<<L[0]<<A[0]<<V[0]<<E[0]<<std::endl;
    }
}

void InitAsgn()
{
    double* X = var; //LuxI
    double* L = var + dev.GetNodeNumber(); //luciferase
    double* A = var + 2*dev.GetNodeNumber(); //AHL
    double* V = var + 3*dev.GetNodeNumber(); //Luminesensor
    double* E = var + 4*dev.GetNodeNumber(); //Illuminance
    for (int it1=0;it1<dev.GetNodeNumber();++it1)
    {
        X[it1]=0.1;
        L[it1]=0.1;
        A[it1]=0.1;
        V[it1]=0.2;
        E[it1]=0;
    }
    std::srand(0);
    for (int it1=0;it1<300;++it1)
    {
        L[std::rand()%dev.GetNodeNumber()]=1;
    }
}

